#!/usr/bin/env python

"""
BaseHTTPRequestHandler adapter.
"""

import WebStack.BaseHTTPRequestHandler
import BaseHTTPServer
from WebStack.Generic import EndOfResponse

class HandlerFactory:

    "A factory class creating WebStack dispatcher objects."

    def __init__(self, resource, authenticator=None):

        """
        Initialise the root application-specific 'resource' and optional
        'authenticator'.
        """

        self.webstack_resource = resource
        self.webstack_authenticator = authenticator

    def __call__(self, request, client_address, server):

        "Act as a factory for the server objects."

        handler = Handler(request, client_address, server, self.webstack_resource,
            self.webstack_authenticator)
        return handler

class Handler(BaseHTTPServer.BaseHTTPRequestHandler):

    "A class dispatching requests to WebStack resources."

    def __init__(self, request, client_address, server, resource, authenticator):

        """
        Initialise the root application-specific 'resource' and 'authenticator'.
        """

        self.webstack_resource = resource
        self.webstack_authenticator = authenticator
        BaseHTTPServer.BaseHTTPRequestHandler.__init__(self, request, client_address, server)

    def handle(self):

        "Dispatch the request to the root application-specific resource."

        # NOTE: Overriding and trimming back the method's functionality.

        self.raw_requestline = self.rfile.readline()
        if not self.parse_request(): # An error code has been sent, just exit
            return

        trans = WebStack.BaseHTTPRequestHandler.Transaction(self)
        if self.webstack_authenticator is None or self.webstack_authenticator.authenticate(trans):
            try:
                self.webstack_resource.respond(trans)
            except EndOfResponse:
                pass
        else:
            trans.set_response_code(401) # Unauthorized
            trans.set_header_value("WWW-Authenticate", '%s realm="%s"' % (
                self.webstack_authenticator.get_auth_type(), self.webstack_authenticator.get_realm()))

        trans.commit()

# vim: tabstop=4 expandtab shiftwidth=4
